#%% imports
import sys
import numpy as np

sys.path.append('../../src/general')

from Puzzle import Puzzle
from PuzzlePiece import PuzzlePiece
from PuzzlePiece import Category
from Side import Side
from Side import SideType

#%% test data 2x2-puzzle
def get_puzzle_2x2():
    
    piece1_side1_outline = np.array([   [0,0,0,1,1,1,0,0,0],
                                        [0,0,1,0,0,0,1,0,0],
                                        [0,0,1,0,0,0,1,0,0],
                                        [0,0,0,1,0,1,0,0,0],
                                        [1,1,1,1,0,1,1,1,1]])
    
    piece1_side2_outline = np.ones(11)
    piece1_side3_outline = np.ones(9)
    
    piece1_side4_outline = np.array([   [0,0,0,0,1,1,0,0,0,0,0],
                                        [0,0,0,1,0,0,1,0,0,0,0],
                                        [0,0,1,0,0,0,0,1,0,0,0],
                                        [0,0,1,0,0,0,0,1,0,0,0],
                                        [0,0,0,1,0,0,0,1,0,0,0],
                                        [1,1,1,1,1,0,1,1,1,1,1]])
    
    piece2_side1_outline = np.array([   [0,0,0,0,1,1,0,0,0,0,0],
                                        [0,0,0,1,0,0,1,0,0,0,0],
                                        [0,0,1,0,0,0,0,1,0,0,0],
                                        [0,0,1,0,0,0,0,1,0,0,0],
                                        [0,0,0,1,0,0,0,1,0,0,0],
                                        [1,1,1,1,1,0,1,1,1,1,1]])
    
    piece2_side2_outline = np.ones(13)
    piece2_side3_outline = np.ones(11)
    
    piece2_side4_outline = np.array([   [0,0,0,1,1,1,1,1,0,0,0,0,0],
                                        [0,0,1,0,0,0,0,0,1,0,0,0,0],
                                        [0,0,1,0,0,0,0,0,1,0,0,0,0],
                                        [0,0,0,1,0,0,0,1,0,0,0,0,0],
                                        [1,1,1,1,0,0,0,1,1,1,1,1,1]])
    
    piece3_side1_outline = np.array([   [0,0,0,1,1,1,0,0,0],
                                        [0,0,1,0,0,0,1,0,0],
                                        [0,0,1,0,0,0,1,0,0],
                                        [0,0,0,1,0,1,0,0,0],
                                        [1,1,1,1,0,1,1,1,1]])
    
    piece3_side2_outline = np.array([   [0,0,0,0,0,0,0,0,0,1,1,0,0,0,0],
                                        [0,0,0,0,0,0,0,0,1,0,0,1,0,0,0],
                                        [0,0,0,0,0,0,0,1,0,0,0,0,1,0,0],
                                        [0,0,0,0,0,0,0,0,1,0,0,1,0,0,0],
                                        [1,1,1,1,1,1,1,1,1,0,1,1,1,1,1]])
    
    piece3_side3_outline = np.ones(9)
    piece3_side4_outline = np.ones(14)
    
    piece4_side1_outline = np.ones(14)
    piece4_side2_outline = np.ones(13)
    
    piece4_side3_outline = np.array([   [0,0,0,0,0,0,0,0,0,1,1,0,0,0,0],
                                        [0,0,0,0,0,0,0,0,1,0,0,1,0,0,0],
                                        [0,0,0,0,0,0,0,1,0,0,0,0,1,0,0],
                                        [0,0,0,0,0,0,0,0,1,0,0,1,0,0,0],
                                        [1,1,1,1,1,1,1,1,1,0,1,1,1,1,1]])
    
    piece4_side4_outline = np.array([   [0,0,0,1,1,1,1,1,0,0,0,0,0],
                                        [0,0,1,0,0,0,0,0,1,0,0,0,0],
                                        [0,0,1,0,0,0,0,0,1,0,0,0,0],
                                        [0,0,0,1,0,0,0,1,0,0,0,0,0],
                                        [1,1,1,1,0,0,0,1,1,1,1,1,1]])
    
    piece1_side1 = Side(0, SideType.HEAD, piece1_side1_outline, 9, 5)
    piece1_side2 = Side(1, SideType.FLAT, piece1_side2_outline, 11, 1)
    piece1_side3 = Side(2, SideType.FLAT, piece1_side3_outline, 9, 1)
    piece1_side4 = Side(3, SideType.HEAD, piece1_side4_outline, 11, 6)
    
    piece2_side1 = Side(0, SideType.HOLE, piece2_side1_outline, 11, 6)
    piece2_side2 = Side(1, SideType.FLAT, piece2_side2_outline, 13, 1)
    piece2_side3 = Side(2, SideType.FLAT, piece2_side3_outline, 11, 1)
    piece2_side4 = Side(3, SideType.HEAD, piece2_side4_outline, 13, 5)
    
    piece3_side1 = Side(0, SideType.HOLE, piece3_side1_outline, 9, 5)
    piece3_side2 = Side(1, SideType.HOLE, piece3_side2_outline, 14, 5)
    piece3_side3 = Side(2, SideType.FLAT, piece3_side3_outline, 9, 1)
    piece3_side4 = Side(3, SideType.FLAT, piece3_side4_outline, 14, 1)
    
    piece4_side1 = Side(0, SideType.FLAT, piece4_side1_outline, 14, 1)
    piece4_side2 = Side(1, SideType.FLAT, piece4_side2_outline, 13, 1)
    piece4_side3 = Side(2, SideType.HEAD, piece4_side3_outline, 14, 5)
    piece4_side4 = Side(3, SideType.HOLE, piece4_side4_outline, 13, 5)
    
    sides_piece1 = np.array([piece1_side1, piece1_side2, piece1_side3, piece1_side4])
    sides_piece2 = np.array([piece2_side1, piece2_side2, piece2_side3, piece2_side4])
    sides_piece3 = np.array([piece3_side1, piece3_side2, piece3_side3, piece3_side4])
    sides_piece4 = np.array([piece4_side1, piece4_side2, piece4_side3, piece4_side4])
    
    piece1 = PuzzlePiece(0, Category.CORNER, 0, 2, '', sides_piece1)
    piece2 = PuzzlePiece(1, Category.CORNER, 1, 1, '', sides_piece2)
    piece3 = PuzzlePiece(2, Category.CORNER, 2, 0, '', sides_piece3)
    piece4 = PuzzlePiece(3, Category.CORNER, 1, 1, '', sides_piece4)
        
    puzzle = Puzzle(2, 2)
    puzzle.set_pieces(np.array([piece1, piece2, piece3, piece4]))
    
    return puzzle

def get_puzzle_3x3():
    piece0_side0_outline = np.array([   [0,0,0,1,1,1,0,0,0],
                                        [0,0,1,0,0,0,1,0,0],
                                        [0,0,1,0,0,0,1,0,0],
                                        [0,0,0,1,0,1,0,0,0],
                                        [1,1,1,1,0,1,1,1,1]])
    
    piece0_side1_outline = np.ones(11)
    piece0_side2_outline = np.ones(9)
    
    piece0_side3_outline = np.array([   [0,0,0,0,1,1,0,0,0,0,0],
                                        [0,0,0,1,0,0,1,0,0,0,0],
                                        [0,0,1,0,0,0,0,1,0,0,0],
                                        [0,0,1,0,0,0,0,1,0,0,0],
                                        [0,0,0,1,0,0,0,1,0,0,0],
                                        [1,1,1,1,1,0,1,1,1,1,1]])
    
    piece1_side0_outline = np.array([   [0,0,0,0,1,1,0,0,0,0,0],
                                        [0,0,0,1,0,0,1,0,0,0,0],
                                        [0,0,1,0,0,0,0,1,0,0,0],
                                        [0,0,1,0,0,0,0,1,0,0,0],
                                        [0,0,0,1,0,0,0,1,0,0,0],
                                        [1,1,1,1,1,0,1,1,1,1,1]])
    
    piece1_side1_outline = np.ones(13)
    
    piece1_side2_outline = np.array([   [0,0,0,0,0,1,1,0,0,0,0],
                                        [0,0,0,0,1,0,0,1,0,0,0],
                                        [1,1,1,1,1,0,0,1,1,1,1]])
    
    piece1_side3_outline = np.array([   [0,0,0,1,1,1,1,1,0,0,0,0,0],
                                        [0,0,1,0,0,0,0,0,1,0,0,0,0],
                                        [0,0,1,0,0,0,0,0,1,0,0,0,0],
                                        [0,0,0,1,0,0,0,1,0,0,0,0,0],
                                        [1,1,1,1,0,0,0,1,1,1,1,1,1]])
    
    piece2_side0_outline = np.array([   [0,0,0,0,0,0,1,1,0,0,0,0],
                                        [0,0,0,0,0,1,0,0,1,0,0,0],
                                        [0,0,0,0,1,0,0,0,0,1,0,0],
                                        [1,1,1,1,1,0,0,0,0,1,1,1]])
    
    piece2_side1_outline = np.array([   [0,0,0,0,0,1,1,0,0,0,0],
                                        [0,0,0,0,1,0,0,1,0,0,0],
                                        [1,1,1,1,1,0,0,1,1,1,1]])
    
    piece2_side2_outline = np.ones(12)
    
    piece2_side3_outline = np.ones(11)
    
    piece3_side0_outline = np.array([   [0,0,0,1,1,1,0,0,0],
                                        [0,0,1,0,0,0,1,0,0],
                                        [0,0,1,0,0,0,1,0,0],
                                        [0,0,0,1,0,1,0,0,0],
                                        [1,1,1,1,0,1,1,1,1]])
    
    piece3_side1_outline = np.array([   [0,0,0,0,0,0,0,0,0,1,1,0,0,0,0],
                                        [0,0,0,0,0,0,0,0,1,0,0,1,0,0,0],
                                        [0,0,0,0,0,0,0,1,0,0,0,0,1,0,0],
                                        [0,0,0,0,0,0,0,0,1,0,0,1,0,0,0],
                                        [1,1,1,1,1,1,1,1,1,0,1,1,1,1,1]])
    
    piece3_side2_outline = np.array([   [0,0,0,1,1,1,0,0,0],
                                        [0,0,1,0,0,0,1,0,0],
                                        [0,0,1,0,0,0,1,0,0],
                                        [1,1,1,0,0,0,1,1,1]])
    
    piece3_side3_outline = np.ones(15)
    
    piece4_side0_outline = np.array([   [0,0,0,0,0,1,1,1,1,1,0,0,0,0,0],
                                        [0,0,0,0,1,0,0,0,0,0,1,0,0,0,0],
                                        [1,1,1,1,1,0,0,0,0,0,0,1,1,1,1]])
    
    piece4_side1_outline = np.array([   [0,0,0,0,0,0,0,1,1,0,0,0,0],
                                        [0,0,0,0,0,1,1,0,0,1,0,0,0],
                                        [0,0,0,0,1,0,0,0,0,1,0,0,0],
                                        [0,0,0,0,1,0,0,0,0,1,0,0,0],
                                        [0,0,0,0,1,0,0,0,0,1,0,0,0],
                                        [1,1,1,1,1,0,0,0,0,1,1,1,1]])
    
    piece4_side2_outline = np.array([   [0,0,0,0,0,0,0,0,0,1,1,0,0,0,0],
                                        [0,0,0,0,0,0,0,0,1,0,0,1,0,0,0],
                                        [0,0,0,0,0,0,0,1,0,0,0,0,1,0,0],
                                        [0,0,0,0,0,0,0,0,1,0,0,1,0,0,0],
                                        [1,1,1,1,1,1,1,1,1,0,1,1,1,1,1]])
    
    piece4_side3_outline = np.array([   [0,0,0,1,1,1,1,1,0,0,0,0,0],
                                        [0,0,1,0,0,0,0,0,1,0,0,0,0],
                                        [0,0,1,0,0,0,0,0,1,0,0,0,0],
                                        [0,0,0,1,0,0,0,1,0,0,0,0,0],
                                        [1,1,1,1,0,0,0,1,1,1,1,1,1]])
    
    piece5_side0_outline = np.array([   [0,0,0,0,0,1,1,1,1,1,0,0,0,0,0],
                                        [0,0,0,0,1,0,0,0,0,0,1,0,0,0,0],
                                        [1,1,1,1,1,0,0,0,0,0,0,1,1,1,1]])
    
    piece5_side1_outline = np.array([   [0,0,0,0,0,0,1,1,0,0,0,0],
                                        [0,0,0,0,0,1,0,0,1,0,0,0],
                                        [0,0,0,0,1,0,0,0,0,1,0,0],
                                        [1,1,1,1,1,0,0,0,0,1,1,1]]) 
    
    piece5_side2_outline = np.ones(15)
    
    piece5_side3_outline = np.array([   [0,0,0,0,0,1,1,1,0,0,0,0],
                                        [1,0,0,0,1,0,0,0,1,0,0,1],
                                        [1,0,0,1,0,0,0,0,0,1,0,1],
                                        [1,1,1,1,0,0,0,0,0,1,1,1]])
    
    piece6_side0_outline = np.array([   [0,0,0,1,1,1,0,0,0],
                                        [0,0,1,0,0,0,1,0,0],
                                        [0,0,1,0,0,0,1,0,0],
                                        [1,1,1,0,0,0,1,1,1]])
    
    piece6_side1_outline = np.array([   [0,0,0,0,1,1,0,0,0,0,0,0],
                                        [0,0,0,1,0,0,1,0,0,0,0,0],
                                        [0,0,0,1,0,0,1,0,0,0,0,0],
                                        [1,1,1,1,0,0,1,1,1,1,1,1]])
    
    piece6_side2_outline = np.ones(9)
    
    piece6_side3_outline = np.ones(12)
    
    piece7_side0_outline = np.array([   [0,0,0,0,1,1,0,0,0,0,0,0],
                                        [0,0,0,1,0,0,1,0,0,0,0,0],
                                        [0,0,0,1,0,0,1,0,0,0,0,0],
                                        [1,1,1,1,0,0,1,1,1,1,1,1]])
    
    piece7_side1_outline = np.array([   [0,0,0,0,0,0,0,1,1,0,0,0,0],
                                        [0,0,0,0,0,1,1,0,0,1,0,0,0],
                                        [0,0,0,0,1,0,0,0,0,1,0,0,0],
                                        [0,0,0,0,1,0,0,0,0,1,0,0,0],
                                        [0,0,0,0,1,0,0,0,0,1,0,0,0],
                                        [1,1,1,1,1,0,0,0,0,1,1,1,1]])
    
    piece7_side2_outline = np.array([   [0,0,0,0,1,1,0,0,0,0,0,0],
                                        [0,0,0,1,0,0,1,0,0,0,0,0],
                                        [1,1,1,0,0,0,0,1,1,1,1,1]])
    
    piece7_side3_outline = np.ones(13)
    
    piece8_side0_outline = np.ones(12)
    
    piece8_side1_outline = np.ones(12)
    
    piece8_side2_outline = np.array([   [0,0,0,0,1,1,0,0,0,0,0,0],
                                        [0,0,0,1,0,0,1,0,0,0,0,0],
                                        [1,1,1,0,0,0,0,1,1,1,1,1]])
                                    
    piece8_side3_outline = np.array([   [0,0,0,0,0,1,1,1,0,0,0,0],
                                        [1,0,0,0,1,0,0,0,1,0,0,1],
                                        [1,0,0,1,0,0,0,0,0,1,0,1],
                                        [1,1,1,1,0,0,0,0,0,1,1,1]])
    
    piece0_side0 = Side(0, SideType.HEAD, piece0_side0_outline, piece0_side0_outline.shape[1], piece0_side0_outline.shape[0])
    piece0_side1 = Side(1, SideType.FLAT, piece0_side1_outline, 1, piece0_side1_outline.shape[0])
    piece0_side2 = Side(2, SideType.FLAT, piece0_side2_outline, 1, piece0_side2_outline.shape[0])
    piece0_side3 = Side(3, SideType.HEAD, piece0_side3_outline, piece0_side3_outline.shape[1], piece0_side3_outline.shape[0])
    
    piece1_side0 = Side(0, SideType.HOLE, piece1_side0_outline, piece1_side0_outline.shape[1], piece1_side0_outline.shape[0])
    piece1_side1 = Side(1, SideType.FLAT, piece1_side1_outline, 1, piece1_side1_outline.shape[0])
    piece1_side2 = Side(2, SideType.HEAD, piece1_side2_outline, piece1_side2_outline.shape[1], piece1_side2_outline.shape[0])
    piece1_side3 = Side(3, SideType.HEAD, piece1_side3_outline, piece1_side3_outline.shape[1], piece1_side3_outline.shape[0])
    
    piece2_side0 = Side(0, SideType.HOLE, piece2_side0_outline, piece2_side0_outline.shape[1], piece2_side0_outline.shape[0])
    piece2_side1 = Side(1, SideType.HOLE, piece2_side1_outline, piece2_side1_outline.shape[1], piece2_side1_outline.shape[0])
    piece2_side2 = Side(2, SideType.FLAT, piece2_side2_outline, 1, piece2_side2_outline.shape[0])
    piece2_side3 = Side(3, SideType.FLAT, piece2_side3_outline, 1, piece2_side3_outline.shape[0])
    
    piece3_side0 = Side(0, SideType.HOLE, piece3_side0_outline, piece3_side0_outline.shape[1], piece3_side0_outline.shape[0])
    piece3_side1 = Side(1, SideType.HOLE, piece3_side1_outline, piece3_side1_outline.shape[1], piece3_side1_outline.shape[0])
    piece3_side2 = Side(2, SideType.HEAD, piece3_side2_outline, piece3_side2_outline.shape[1], piece3_side2_outline.shape[0])
    piece3_side3 = Side(3, SideType.FLAT, piece3_side3_outline, 1, piece3_side3_outline.shape[0])
    
    piece4_side0 = Side(0, SideType.HEAD, piece4_side0_outline, piece4_side0_outline.shape[1], piece4_side0_outline.shape[0])
    piece4_side1 = Side(1, SideType.HOLE, piece4_side1_outline, piece4_side1_outline.shape[1], piece4_side1_outline.shape[0])
    piece4_side2 = Side(2, SideType.HEAD, piece4_side2_outline, piece4_side2_outline.shape[1], piece4_side2_outline.shape[0])
    piece4_side3 = Side(3, SideType.HOLE, piece4_side3_outline, piece4_side3_outline.shape[1], piece4_side3_outline.shape[0])
    
    piece5_side0 = Side(0, SideType.HOLE, piece5_side0_outline, piece5_side0_outline.shape[1], piece5_side0_outline.shape[0])
    piece5_side1 = Side(1, SideType.HEAD, piece5_side1_outline, piece5_side1_outline.shape[1], piece5_side1_outline.shape[0])
    piece5_side2 = Side(2, SideType.FLAT, piece5_side2_outline, 1, piece5_side2_outline.shape[0])
    piece5_side3 = Side(3, SideType.HOLE, piece5_side3_outline, piece5_side3_outline.shape[1], piece5_side3_outline.shape[0])
    
    piece6_side0 = Side(0, SideType.HOLE, piece6_side0_outline, piece6_side0_outline.shape[1], piece6_side0_outline.shape[0])
    piece6_side1 = Side(1, SideType.HOLE, piece6_side1_outline, piece6_side1_outline.shape[1], piece6_side1_outline.shape[0])
    piece6_side2 = Side(2, SideType.FLAT, piece6_side2_outline, 1, piece6_side2_outline.shape[0])
    piece6_side3 = Side(3, SideType.FLAT, piece6_side3_outline, 1, piece6_side3_outline.shape[0])
    
    piece7_side0 = Side(0, SideType.HEAD, piece7_side0_outline, piece7_side0_outline.shape[1], piece7_side0_outline.shape[0])
    piece7_side1 = Side(1, SideType.HEAD, piece7_side1_outline, piece7_side1_outline.shape[1], piece7_side1_outline.shape[0])
    piece7_side2 = Side(2, SideType.HOLE, piece7_side2_outline, piece7_side2_outline.shape[1], piece7_side2_outline.shape[0])
    piece7_side3 = Side(3, SideType.FLAT, piece7_side3_outline, 1, piece7_side3_outline.shape[0])
    
    piece8_side0 = Side(0, SideType.FLAT, piece8_side0_outline, 1, piece8_side0_outline.shape[0])
    piece8_side1 = Side(1, SideType.FLAT, piece8_side1_outline, 1, piece8_side1_outline.shape[0])
    piece8_side2 = Side(2, SideType.HEAD, piece8_side2_outline, piece8_side2_outline.shape[1], piece8_side2_outline.shape[0])
    piece8_side3 = Side(3, SideType.HEAD, piece8_side3_outline, piece8_side3_outline.shape[1], piece8_side3_outline.shape[0])
    
    sides_piece0 = np.array([piece0_side0, piece0_side1, piece0_side2, piece0_side3])
    sides_piece1 = np.array([piece1_side0, piece1_side1, piece1_side2, piece1_side3])
    sides_piece2 = np.array([piece2_side0, piece2_side1, piece2_side2, piece2_side3])
    sides_piece3 = np.array([piece3_side0, piece3_side1, piece3_side2, piece3_side3])
    sides_piece4 = np.array([piece4_side0, piece4_side1, piece4_side2, piece4_side3])
    sides_piece5 = np.array([piece5_side0, piece5_side1, piece5_side2, piece5_side3])
    sides_piece6 = np.array([piece6_side0, piece6_side1, piece6_side2, piece6_side3])
    sides_piece7 = np.array([piece7_side0, piece7_side1, piece7_side2, piece7_side3])
    sides_piece8 = np.array([piece8_side0, piece8_side1, piece8_side2, piece8_side3])
    
    piece0 = PuzzlePiece(0, Category.CORNER, 0, 2, '', sides_piece0)
    piece1 = PuzzlePiece(1, Category.EDGE, 1, 2, '', sides_piece1)
    piece2 = PuzzlePiece(2, Category.CORNER, 2, 0, '', sides_piece2)
    piece3 = PuzzlePiece(3, Category.EDGE, 2, 1, '', sides_piece3)
    piece4 = PuzzlePiece(4, Category.INNER, 2, 2, '', sides_piece4)
    piece5 = PuzzlePiece(5, Category.EDGE, 2, 1, '', sides_piece5)
    piece6 = PuzzlePiece(6, Category.CORNER, 2, 0, '', sides_piece6)
    piece7 = PuzzlePiece(7, Category.EDGE, 1, 2, '', sides_piece7)
    piece8 = PuzzlePiece(8, Category.CORNER, 0, 2, '', sides_piece8)
    
    puzzle = Puzzle(3, 3)
    puzzle.set_pieces(np.array([piece0, piece1, piece2, piece3, piece4, piece5, piece6, piece7, piece8]))
    
    return puzzle