# -*- coding: utf-8 -*-
"""
Created on Mon Apr 22 11:27:26 2024

@author: Jessica
"""

# %% Imports
import numpy as np
from PuzzlePiece import PuzzlePiece
from PuzzlePiece import Category
from Side import Side
from Side import SideType

# %% Class Puzzle
class Puzzle:
    """
    A class used to represent a puzzle. 
    The puzzle pieces are not necessarily ordered.
    
    Attributes
    ----------
    __size : np.array
        an array that contains the width and length and represents
        the size of the puzzle
    __width : int
        the width of the puzzle
    __height : int
        the height of the puzzle
    __pieces : np.array
        an array that contains all the puzzle pieces
    
    Methods
    ------- 
    set_piece(piece : PuzzlePiece, idx : int)
    get_edge_pieces(is_sorted : bool) : np.array
    get_inner_pieces(is_sorted : bool) : np.array
    get_corners(is_sorted : bool) : np.array
    
    """
    
    def __init__(self, width, height):
        """
        Initialises an instance of Puzzle.
        
        Parameters
        ----------
        width : int
            the width of the puzzle (number of puzzle pieces)
        height : int
            the height of the puzzle (number of puzzle pieces)
            
        """
        
        self.__size = [width, height]
        self.__width = width
        self.__height = height
        self.__pieces = np.empty(shape = (width * height), dtype = object) #np.ndarray(shape = self.__size, dtype = object) #np.zeros(self.size)   # TODO: ggf. anpassen
    
    def get_size(self):
        return self.__size
    
    def get_width(self):
        return self.__width
    
    def get_height(self):
        return self.__height
    
    def get_pieces(self):
        return self.__pieces
    
    def set_pieces(self, pieces):
        self.__pieces = pieces
    
    def set_piece(self, piece: PuzzlePiece, idx):
        """
        Sets one piece in the array pieces.

        Parameters
        ----------
        piece : PuzzlePiece
            a puzzle piece
        idx : int
            index

        Returns
        -------
        None.

        """

        self.__pieces[idx] = piece
        
    def get_piece_by_id(self, id):
        pass # TODO
    
    def get_edge_pieces(self, is_sorted = False):
        """
        Gets the edge pieces.

        Parameters
        ----------
        is_sorted : boolean, optional
            decides if sorted pieces should be returned. The default is False.

        Returns
        -------
        edges : np.array
           an array that contains all the edge pieces

        """
        
        edges = [p for p in self.pieces if p.category == Category.EDGE and p.is_sorted == is_sorted]
        return edges
    
    def get_inner_pieces(self, is_sorted = False):
        """
        Gets the inner pieces.

        Parameters
        ----------
        is_sorted : boolean, optional
            decides if sorted pieces should be returned. The default is False.

        Returns
        -------
        inner : np.array
            an array that contains all the inner pieces

        """

        inner = [p for p in self.pieces if p.category == Category.INNER and p.is_sorted == is_sorted]
        return inner
    
    def get_corners(self, is_sorted = False):
        """
        Gets the corners

        Parameters
        ----------
        is_sorted : boolean, optional
            decides if sorted pieces should be returned. The default is False.

        Returns
        -------
        corners : np.array
            an array that contains all the inner pieces

        """
        
        corners = [p for p in self.pieces if p.category == Category.CORNER and p.is_sorted == is_sorted]
        return corners
                
    size = property(get_size)
    width = property(get_width)
    height = property(get_height)
    pieces = property(get_pieces, set_pieces)