# -*- coding: utf-8 -*-
"""
Created on Mon Apr 22 11:51:05 2024

@author: Jessica
"""

# %% Imports
from enum import Enum

# %% Enum Side Type
class SideType(Enum):
    """
    An enumeration representing different types of sides.

    The possible types are:
        - FLAT: A flat side.
        - HOLE: A hole (notch).
        - HEAD: A head (bulge).
        
    """
    
    FLAT = 1
    HOLE = 2
    HEAD = 3

# %% Class Side
class Side:
    """
    A class used to represent a side of a puzzle piece.
    
    Attributes
    ----------
    __id : int
        id of the side
    __side_type : SideType
        a value of the enumeration SideType
    __outline : np.array
        an array that contains the outline of the side
    __width : int
        the width of the side
    __height : int
        the height of the side
    __is_sorted : bool
        decides if the side is already sorted
    
    """
    
    def __init__(self, id, side_type: SideType, outline, width, height):
        """
        
        Parameters
        ----------
        side_type : SideType
            a value of the enumeration SideType
        outline : np.array
            an array that contains the outline of the side
        width : int
            the width of the side
        height : int
            the height of the side

        """
        self.__id = id
        self.__side_type = side_type
        self.__outline = outline
        self.__width = width
        self.__height = height
        self.__is_sorted = False
        
    def get_id(self):
        return self.__id
    
    def get_side_type(self):
        return self.__side_type
    
    def set_side_type(self, side_type):
        if side_type in SideType:
            self.__side_type = side_type
        else:
            print("Invalid SideType")
        
    def get_outline(self):
        return self.__outline
    
    def set_outline(self, outline):
        self.__outline = outline
        
    def get_width(self):
        return self.__width
    
    def set_width(self, width):
        self.__width = width
    
    def get_height(self):
        return self.__height
    
    def set_height(self, height):
        self.__height = height
    
    def get_is_sorted(self):
        return self.__is_sorted
    
    def set_is_sorted(self, is_sorted):
        self.__is_sorted = is_sorted
        
    id = property(get_id)
    side_type = property(get_side_type, set_side_type)
    outline = property(get_outline, set_outline)
    width = property(get_width, set_width)
    height = property(get_height, set_height)
    is_sorted = property(get_is_sorted, set_is_sorted)
    